@@ -11,6 +11,7 @@ pub async fn connect_tls<S, T>(
     mut stream: S,
     mode: SslMode,
     tls: T,
+    has_hostname: bool,
 ) -> Result<MaybeTlsStream<S, T::Stream>, Error>
 where
     S: AsyncRead + AsyncWrite + Unpin,
@@ -39,6 +40,10 @@ where
         }
     }
 
+    if !has_hostname {
+        return Err(Error::tls("no hostname provided for TLS handshake".into()));
+    }
+
     let stream = tls
         .connect(stream)
         .await
